package com.jiuqi.util.patterns;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

class PlaceholderPattern implements IPattern {
   private List items = new ArrayList();
   private String expression;
   private boolean isVariable;

   public PlaceholderPattern() {
   }

   public String toString() {
      StringBuffer buffer = new StringBuffer();
      if (this.isVariable) {
         buffer.append("${");
      }

      for(int i = 0; i < this.size(); ++i) {
         buffer.append(this.get(i).toString());
      }

      if (this.isVariable) {
         buffer.append("}");
      }

      return buffer.toString();
   }

   public boolean isVariable() {
      return this.isVariable;
   }

   public void setIsVariable(boolean value) {
      this.isVariable = value;
   }

   public void parse(String expression) throws PatternException {
      this.clear();
      this.expression = expression;
      if (expression != null) {
         int cur = 0;
         int start = 0;
         int stackSize = 0;
         boolean startVar = false;

         while(cur < expression.length()) {
            switch (this.charOf(cur)) {
               case '$':
                  if (this.charOf(cur + 1) == '{') {
                     if (startVar) {
                        ++stackSize;
                     } else {
                        this.addConst(start, cur);
                        start = cur;
                        startVar = true;
                        stackSize = 0;
                     }

                     cur += 2;
                  } else {
                     ++cur;
                  }
                  break;
               case '}':
                  if (startVar) {
                     if (stackSize == 0) {
                        this.addVariable(start + 2, cur);
                        startVar = false;
                        start = cur + 1;
                     } else {
                        --stackSize;
                     }
                  }

                  ++cur;
                  break;
               default:
                  ++cur;
            }
         }

         if (cur == expression.length() && cur > start) {
            this.addConst(start, cur);
         }

      }
   }

   private char charOf(int index) {
      return index >= 0 && index < this.expression.length() ? this.expression.charAt(index) : '\u0000';
   }

   private void addConst(int start, int cur) {
      if (start < cur) {
         String s = this.expression.substring(start, cur);
         ConstPattern p = new ConstPattern(s);
         this.add(p);
      }
   }

   private void addVariable(int start, int cur) throws PatternException {
      if (start < cur) {
         String s = this.expression.substring(start, cur);
         int p1 = s.indexOf("${");
         int p2 = s.lastIndexOf(125);
         if (p1 >= 0 && p1 < p2) {
            PlaceholderPattern p = new PlaceholderPattern();
            p.parse(s);
            p.setIsVariable(true);
            this.add(p);
         } else {
            VariablePattern p = new VariablePattern();
            p.parse(s);
            this.add(p);
         }

      }
   }

   public void add(IPattern pattern) {
      this.items.add(pattern);
   }

   public void add(int index, IPattern pattern) {
      this.items.add(index, pattern);
   }

   public void clear() {
      this.items.clear();
      this.expression = null;
      this.isVariable = false;
   }

   public IPattern get(int index) {
      return (IPattern)this.items.get(index);
   }

   public String getExpression() {
      return this.toString();
   }

   public int getLength() {
      return 0;
   }

   public int getType() {
      return 1;
   }

   public int indexOf(IPattern pattern) {
      return this.items.indexOf(pattern);
   }

   public void remove(int index) {
      this.items.remove(index);
   }

   public void setExpression(String value) {
      try {
         this.parse(value);
      } catch (PatternException var2) {
         this.clear();
      }

   }

   public void setLength(int value) {
   }

   public int size() {
      return this.items.size();
   }

   public int match(String s, int start, Map values) {
      if (this.isVariable) {
         return -1;
      } else {
         int cur = 0;

         for(int next = start; cur < this.size(); ++cur) {
            switch (this.get(cur).getType()) {
               case 0:
                  next = this.get(cur).match(s, start, values);
                  if (next == -1) {
                     return -1;
                  }
                  break;
               case 1:
                  String value;
                  if (this.get(cur).getLength() > 0) {
                     next = this.get(cur).match(s, start, values);
                     if (next == -1) {
                        return -1;
                     }

                     value = s.substring(start, next);
                     values.put(this.get(cur).getExpression(), value);
                  } else if (cur >= this.size() - 1) {
                     value = s.substring(start);
                     values.put(this.get(cur).getExpression(), value);
                     next = s.length();
                  } else if (this.get(cur + 1).getType() == 0) {
                     next = s.indexOf(this.get(cur + 1).getExpression(), start + 1);
                     if (next == -1) {
                        return -1;
                     }

                     value = s.substring(start, next);
                     values.put(this.get(cur).getExpression(), value);
                  } else {
                     int backLen = this.statNextLength(cur);
                     if (backLen == -1 || start >= s.length() - backLen) {
                        return -1;
                     }

                     next = s.length() - backLen;
                     String value = s.substring(start, next);
                     values.put(this.get(cur).getExpression(), value);
                  }
            }

            start = next;
         }

         return start;
      }
   }

   private int statNextLength(int cur) {
      int len = 0;

      for(int i = cur + 1; i < this.size(); ++i) {
         switch (this.get(i).getType()) {
            case 0:
               len += this.get(i).getLength();
               break;
            case 1:
               if (this.get(i).getLength() <= 0) {
                  return -1;
               }

               len += this.get(i).getLength();
               break;
            default:
               return -1;
         }
      }

      return len;
   }

   public String replace(ITranslator translator) {
      StringBuffer buffer = new StringBuffer();

      for(int i = 0; i < this.size(); ++i) {
         buffer.append(this.get(i).replace(translator));
      }

      if (this.isVariable) {
         String[] ret = translator.translate(buffer.toString(), 0);
         if (ret == null) {
            return buffer.toString();
         } else {
            return ret.length == 0 ? "" : ret[0];
         }
      } else {
         return buffer.toString();
      }
   }
}
